c========================================================================o
c University of Colorado, Phys. Dept., CIEI                              o
c Program d2tracerdn v1.1.1, Created: 07/21/2002                         o
c          Author: M.P.Barmin                                            o
c          Phone: (303) 735-1850                                         o
c          Fax:   (303) 492-7935                                         o
c          Email: barmin@colorado.edu                                    o
c The D2tracerdn is a program designed for the numerical modelling       o
c of refracted P or S travel times in 3D laterally inhomogeneous         o
c media with curved interfaces by the ray method. D2tracerdn performs    o
c 2D ray tracing across 3D layered model on a sphere using 2D            o
c crossection profile. Program constructs SSSC's for P(S) waves relative o
c to 1D reference model or absolute 3D travel time table (TTT)           o
c in the vicinity of 22 degree surrounding an arbitrary station for      o
c not more then ten arbitrary depths starting from free surface.         o
c D2tracerdn is based on the well known  Psencik-Cerveny ray method for  o
c 2D laterally varying layered structures.                               o
c========================================================================o
      implicit none
c --- common block definitions ---
      integer*4 nz,nn(100),ind(101,100,2)
      real*8    delta,wmod(101,100,2),ff(100),fl(100)
      common/mod2/delta,wmod,ff,fl,nn,ind,nz
c ---
      real*8    geo,r0,pi,rad,h(10),hs(10)
      integer*4 nhs,max_delta
      common/const/geo,r0,pi,rad,h,hs,nhs,max_delta
c ---
      integer*4 i2,j2
      common/work/i2,j2
c ---
      integer*4 idbg,iaz_i,iaz_e,iaz_st,iaz_c,igraph,igrc,igeoc,npid
      real*8    azi,sfi,slam,elev
      character*8 sta_name,surr_name,tables
      character*8 spid
      common/all/azi,sfi,slam,elev,spid,idbg,iaz_i,iaz_e,iaz_st,iaz_c,
     *sta_name,surr_name,tables,igraph,igrc,igeoc,npid
c ---
      character*160 model_cu,model_iasp,out_dir
      common/fnames/ model_cu,model_iasp,out_dir
c ---
      integer*4 nlayers,npoints,mnn(43000),mind(100,43000,2),
     *          itab(179,361)
      real*8    gff(43000),gfl(43000)
      real*4    gmod(100,43000,2)
      common/modcu/ nlayers,npoints,gff,gfl,gmod,mnn,itab,mind
c ---
      integer*4   nlayers1,net1,iwave1,igroup(100)
      integer*4   nng,ngrno(100),ngrdev(100)
      real*8      st_rk,dnet1
      character*8 p_phname(100),s_phname(100),name(100)
      character*1 c_wave(100),c_head(100)
      common/prm/st_rk,dnet1,nlayers1,iwave1,net1,
     *nng,ngrno,ngrdev,name,
     *igroup,p_phname,s_phname,c_wave,c_head
c --- type definitions ---
      real*8    a0,a1,a2,a3,zf,zl,dnet
      integer*4 iargc,lnblnk,lb,lb1,lb2,lb3,getpid
      integer*4 i,i1j,i1j1,ij,ij1,iwave,k,kk,l,lll,
     *narg,n_lay,nf,nl,net
      character*20 ppp,wave
      character*160 par_file,f_surf,cmd,spid_n(6)
      real*8    take_off(20)
      integer*4 igrp,idev,ndev
      character*8 fname,spida,m_name
      character*1 cc_w,cc_h
c ---
      geo=0.993277d0
      r0=6371.0d0
      pi=datan(1.0d0)*4.0d0
      rad=pi/180.0d0
      i2=0
      j2=0
c Get process id =======================
      npid=getpid()
      write(spida,'(i8)') npid
      do i=1,8
        if(spida(i:i).ne.' ') goto 777
      enddo
 777  spid=spida(i:8)
      npid=lnblnk(spid)
      narg=iargc()
      if(narg.ne.1.and.narg.ne.6) then
      write(*,*) 'Usage: d2tracerdn parameter_file [sta_name surr_name s_lat s_lon elev]'
      stop 
      endif
c
c Read parameters from parameter file
c
      call getarg(1,par_file)
      call set_defaults(par_file,wave,m_name)
      do i=1,nhs
        h(i)=-r0*dlog(1.0d0-hs(i)/r0)
      enddo
      if(igeoc.eq.1) sfi=datan(geo*dtan(rad*sfi))/rad
      nlayers=nlayers1
      net=net1
      dnet=dnet1
      iwave=iwave1
c
c read command line arguments =====================
c
      if(narg.ge.2) then
          call getarg(2,sta_name)
          call getarg(3,surr_name)
          call getarg(4,ppp)
          read(ppp,*) sfi
          if(igeoc.eq.1) sfi=datan(geo*dtan(rad*sfi))/rad
          call getarg(5,ppp)
          read(ppp,*) slam
          if(slam.lt.0.0d0) slam=slam+360.0d0
          call getarg(6,ppp)
          read(ppp,*) elev
      endif
c
c Read 3-D model in common/model/ ============================
c
      call read_model(iwave,net)
      n_lay=mnn(1)
c
c Create SSSC's filenames =================================
c
      lb1=lnblnk(sta_name)
      lb2=lnblnk(out_dir)
      call group()
      do i=1,nlayers
        call lay_dev(i,igrp,idev,ndev,fname,cc_w,cc_h)
        write(*,*) i,igrp,idev,ndev,fname,cc_w,cc_h
      enddo
c ---
      do i=1,nng
        idev=ngrdev(i)
        lb=lnblnk(name(i))
        lb3=lnblnk(m_name)
        f_surf=out_dir(1:lb2)//'/'//sta_name(1:lb1)//'.'//
     *          name(i)(1:lb)//'.'//m_name(1:lb3)//'.dat'
        open(20+idev,file=f_surf,status='unknown')
        lb=lnblnk(f_surf)
        write(*,*) 20+idev,f_surf(1:lb)
      enddo
c
c Main loop ========================================
c
      nz=22
      do iaz_c=iaz_i,iaz_e,iaz_st
      azi=iaz_c
      igrc=0
      call path_az(2.0d0,sfi,slam,azi,delta,nz,ff,fl)
c Debug: geocentrical profile coordinates ================
      if(idbg.gt.0) then
        do i=1,nz
          write(*,*) i,ff(i),fl(i)
        enddo
      endif
c Debug end ==============================================
        if(net.ne.2.and.net.ne.1) net=2
        do k=1,nz
          nn(k)=n_lay
          nf=ff(k)+90.0d0
          nf=nf/net*net
          if(nf.lt.1.or.nf.gt.179) goto 7
          nl=fl(k)
          nl=nl/net*net+1
          if(nl.lt.1.or.nl.gt.361) goto 7
          zf=(ff(k)-(nf-90))/dnet 
          zl=(fl(k)-(nl-1))/dnet 
          ij=itab(nf,nl)
          i1j=itab(nf+net,nl)
          ij1=itab(nf,nl+net)
          i1j1=itab(nf+net,nl+net)
          goto 8
    7     ij=0
          i1j=0
          ij1=0
          i1j1=0
    8     continue
          do kk=1,n_lay
            ind(kk,k,1)=kk
            ind(kk,k,2)=mind(kk,1,2)
            if(ij.eq.0.or.i1j.eq.0.or.ij1.eq.0.or.i1j1.eq.0) then
            if(k-1.eq.0) then
              write(*,'("(E003) Starting point:",2f10.4," is outside model")')
     *        sfi,slam
              write(*,1003) wave,sfi,slam,azi
 1003         format('(I001) ',a4,'Sour:',2f8.2,', Azim:',f8.2)
              goto 9999
            endif
              do lll=1,2
                wmod(kk,k,lll)=wmod(kk,k-1,lll)
              enddo
            else
            do lll=1,2
              a0=gmod(kk,ij,lll)
              a1=gmod(kk,i1j,lll)
              a2=gmod(kk,ij1,lll)
              a3=gmod(kk,i1j1,lll)
              wmod(kk,k,lll)=
     *        a0+(a1-a0)*zf+(a2-a0)*zl+(a0+a3-a1-a2)*zf*zl
            enddo
            endif
          enddo
        enddo
        call psinp(iwave,1,2,n_lay,2.0d0,take_off,nlayers)
  999 continue
c delete temporary files ================================
      if(idbg.eq.0) then
        do i=1,nng+1
          write(spida,'(i1)') i
          spid_n(i)='tmp.'//spid(1:npid)//'.'//spida(1:1)
        enddo
        l=lnblnk(spid_n(1))+1
        cmd='/bin/rm '//spid_n(1)(1:l)//spid_n(2)(1:l)//spid_n(3)(1:l)//
     *            spid_n(4)(1:l)
        call system(cmd)
        cmd='/bin/rm tmp.'//spid(1:npid)//'.seis.data '//
     *      'tmp.'//spid(1:npid)//'.se.out '//
     *      'tmp.'//spid(1:npid)//'.A '//
     *      'tmp.'//spid(1:npid)//'.B '
        call system(cmd)
      endif
      if(igraph.gt.0) then
        do i=1,nng+1
          close(40+i)
        enddo
      endif
      enddo
 9999 continue
      do i=1,nng
        idev=ngrdev(i)
        close(20+idev)
      enddo
      end
