c====================================================
c iasp91_read  read IASP91 travel time table
c for phases and dephts defined in reference 1D model
c distance: 0 - 2250 km by step 25 km
c====================================================
      subroutine iasp91_read
      implicit none
c --- type definitions ---
      character*8 phase, sta1,sta2
      real*8    hs_ias(10)
      real*8    dum(4)
      integer*4 i,j,kk,l,mhs
c ---
      integer*4 n_phase
      real*8    t_iasp91(101,11,19)
      character*8 ph(19)
      common/iasp91/t_iasp91,ph,n_phase
c --- common blocks ---
      character*160 model_cu,model_iasp,out_dir
      common/fnames/ model_cu,model_iasp,out_dir
c ---
      real*8    geo,r0,pi,rad,h(10),hs(10)
      integer*4 nhs,max_delta
      common/const/geo,r0,pi,rad,h,hs,nhs,max_delta
c ---
      integer*4 idbg,iaz_i,iaz_e,iaz_st,iaz_c,igraph,igrc,igeoc,npid
      integer*4 mdelta
      real*8    azi,sfi,slam,elev
      character*8 sta_name,surr_name,tables
      character*8 spid
      common/all/azi,sfi,slam,elev,spid,idbg,iaz_i,iaz_e,iaz_st,iaz_c,
     *sta_name,surr_name,tables,igraph,igrc,igeoc,npid
c ---
c     data ph/'Pg','Pn','P','Lg','Sn','S'/
      if(tables.eq.'TTT') return
      open(7,file=model_iasp,status='old')
      kk=0
   2  kk=kk+1
      read(7,'(4f12.5,2i5,1x,a8,1x,a8,1x,a8)',end=9) 
     *      (dum(l),l=1,4),mhs,mdelta,sta1,sta2,phase
      read(7,'(f7.1,9f9.3)') (hs_ias(l),l=1,mhs)
      if(mhs.ne.nhs) then
         write(*,*)'(E100) Number of depths in D2tracerdn',nhs,
     *' does not correspond',' number of depth', mhs,' in reference table',
     *' for phase: ',phase
         stop
      endif
      if(max_delta.ne.mdelta) then
         write(*,*)'(E104) Different delta for D2tracerdn and reference model'
         stop
      endif
      do i=1,nhs
         if(dabs(hs_ias(i)-hs(i)).gt.0.01) then
             write(*,*) '(E101) Different depths in ref. tables',
     *                  ' and D2tracerdn for phase: ', phase
             do j=1,nhs
                write(*,*) j,hs_ias(j),hs(j)
             enddo
             stop
         endif
      enddo
      ph(kk)=phase
        do i=1,mdelta+1
          read(7,*) (t_iasp91(i,l,kk),l=1,mhs+1)
        enddo
      goto 2
   9  close(7)
      n_phase=kk
      return
      end
c======================================================
c Get IASP91 string for fixed phase and distance
c======================================================
      subroutine iasp91_get(delta,phase,tab)
      implicit none
c --- type definitions ---
      real*8    delta,tab(7)
      integer*4 i,idel,iphase
      character*8 phase
c --- common blocks ---
      character*8 ph(19)
      integer*4 n_phase
      real*8    t_iasp91(101,11,19)
      common/iasp91/t_iasp91,ph,n_phase
c ---
      real*8    geo,r0,pi,rad,h(10),hs(10)
      integer*4 nhs,max_delta
      common/const/geo,r0,pi,rad,h,hs,nhs,max_delta
c ---
      integer*4 idbg,iaz_i,iaz_e,iaz_st,iaz_c,igraph,igrc,igeoc,npid
      real*8    azi,sfi,slam,elev
      character*8 sta_name,surr_name,tables
      character*8 spid
      common/all/azi,sfi,slam,elev,spid,idbg,iaz_i,iaz_e,iaz_st,iaz_c,
     *sta_name,surr_name,tables,igraph,igrc,igeoc,npid
c ---
      idel=(delta+5.0d0)/25.0d0+1
      if(tables.ne.'TTT') then
          iphase=0
          do i=1,n_phase
            if(ph(i).eq.phase) iphase=i
          enddo
          if(iphase.eq.0) then
            write(*,*) 'No phase name',phase,'in reference table'
            stop
          endif
          do i=1,nhs+1
            tab(i)=t_iasp91(idel,i,iphase)
          enddo
      else
          tab(1)=idel*25.0d0
          do i=2,nhs+1
             tab(i)=0.0d0
          enddo
      endif
      return
      end
c===============
c TEST
c===============
c     real*8 distance,tab(7)
c     call iasp91_read
c 1   write(*,*) 'Enter ==> iphase,distance'
c     read(*,*) iphase,distance
c     call iasp91_get(distance,iphase,tab)
c     write(*,*)tab
c     goto 1
c     end
