c******************************************************************
c
c     Ray tracing by the Runge-Kutta method
c    ======================================= 
c     x0,z0 - initial point of the ray
c     t0    - initial time
c     fi0   - initial angle
c     x,z   - point at which the ray computation is terminated
c     t,fi  - corresponding time and angle
c     tmax  - the maximum permitted time, here tmax=10000.
c     dt    - step of integration
c     ac    - accuracy of integration, recommended (0.0001 -0.001 ??)
c-------- control parameters ---------------------------------------
c     ind/ray/ - starting layer
c     kc/cod/  - =1, should strike interface below the source
c     kref/cod/ - the number of simple element in the ray
c******************************************************************
      subroutine ray2(x0,z0,t0,fi0,x,z,t,fi,tmax,dt,ac)
      implicit none
c ---
      integer*4 n_int,n_intz
      parameter (n_int=20,n_intz=50)
c --- type definitions ---
      real*8    y(4),dery(4),din(4),prm(5),aux(8,4),vel(6)
      real*8    ac,cs,dt,fi,fi0,sn,t,t0,tmax,v,x,x0,z,z0
      integer*4 i,ihlf,isour
c --- commons ---
      real*8    a1,b1,c1,d1,x1,brd
      integer*4 iii,npnt,nint
      common/intrf/a1(n_intz,n_int),b1(n_intz,n_int),c1(n_intz,n_int),
     *d1(n_intz,n_int),x1(n_intz,n_int),brd(2),iii(n_intz,n_int),
     *npnt(n_int),nint
c ---
      integer*4 intr,int1,iout,kre,irefr,lay,itype,nder,iprint,mprint,ntr,imet
      common/auxi/intr,int1,iout,kre,irefr,lay,itype,nder,iprint,mprint,ntr,imet
c ---
      real*8    ay,ds
      integer*4 kint,mreg,n,iref,ind,ind1
      common/ray/ay(12,10000),ds(11,50),kint(50),mreg,n,iref,ind,ind1
c ---
      integer*4 code,kref,kc,k_beg,k_end,k_step,k_sig
      common/cod/code(50),kref,kc,k_beg,k_end,k_step,k_sig
c ---
      y(1)=x0
      y(2)=z0
      irefr=0
      kre=kref
      if(kc.eq.0)kre=0
      n=0
      iref=1
      iout=0
      isour=ind
c
c     Determination of initial conditions for the runge-kutta procedure
c
      lay=isour
      if(isour.ne.iabs(code(1)))ind=14
      if(isour.ne.iabs(code(1)))return
      itype=isign(1,code(1))
      y(3)=0.d0
      y(4)=0.d0
      call veloc(y,vel)
      v=vel(1)
      y(3)=dcos(fi0)/v
      y(4)=dsin(fi0)/v
      ind=0
      ind1=0
      prm(1)=t0
      prm(2)=tmax
      prm(3)=dt
      prm(4)=ac
      t=prm(1)
      do 21 i=1,4
   21 din(i)=.25d0
    9 do 10 i=1,4
   10 dery(i)=din(i)
c
c     Computation of the ray
c
      call rkgs(prm,y,dery,4,ihlf,aux)
      if(ihlf.eq.11)ind=5
      if(ihlf.eq.12)ind=6
      if(ihlf.eq.13)ind=7
      if(ind.ge.5.and.ind.le.7)return
      if(ind.eq.20)return
      if(ind.eq.21)return
      if(dabs(prm(5)).gt..0001d0) go to 11
c
c     Interuption of the computation of the ray due to (t.ge.prm(2))
c     if prm(2).eq.tmax - termination of the computation
c     if prm(2).ne.tmax - the computation continues further on with
c     the regular time step along the ray
c
      t=ay(1,n)
      if(t.ge.tmax)go to 15
      go to 14
   11 if(dabs(prm(5)-1.1d0).gt..0001d0) go to 13
c
c     Loop for the iterative determination of the point of incidence
c     (not used in this version)
c
      prm(1)=ay(1,n)
      prm(3)=prm(3)/(2.d0**(ihlf+1))
      do 12 i=1,4
   12 y(i)=ay(i+1,n)
      t=ay(1,n)
      n=n-1
      go to 9
   13 x=y(1)
      z=y(2)
      t=ay(1,n)
      if(dabs(prm(5)-2.d0).gt..0001d0.and.irefr.eq.1)ind1=-ind1
      if(dabs(prm(5)-2.d0).gt..0001d0)go to 20
c
c     Integration from the point of reflection/transmission
c     to the closest point of the ray corresponding to the regular
c     time step along the ray
c
      prm(1)=ay(1,n)
      i=int((prm(1)-t0)/dt)
      prm(2)=(i+1)*dt+t0
      go to 9
   14 prm(1)=prm(2)
      prm(2)=tmax
      prm(3)=dt
      n=n-1
      go to 9
   15 ind=12
      x=y(1)
      z=y(2)
      if(irefr.eq.1)ind1=-ind1
c
   20 v=ay(6,n)
      cs=y(3)*v
      sn=y(4)*v
      fi=datan2(sn,cs)
      return
      end
c******************************************************************
c     External routine in the Runge-Kutta ray method.
c     Performs various computations at each point of the ray
c
c     iref/ray/ - reference to ds file with reflection points
c******************************************************************
      subroutine outp(x,y,dery,ihlf,ndim,prmt)
      implicit none
c ---
      integer*4 n_int,n_intz,n_lay
      parameter (n_int=20,n_intz=50,n_lay=19)
c --- type definitions ---
      real*8    x,y(4),dery(4),prmt(5)
      integer*4 ihlf,ndim
c --- commons ---
      real*8    yold(2),yint(2),vel(6),y1(2)
      real*8    aindex,anyx,anyz,aux,aux1,aux2,auxx,auxz,avert,ay4,ay5
      real*8    px,pz,rc,v1,v2,vn,vrt,vvv,x2,xdif,yan,ydif,zint,zint1
      integer*4 i,i11,iac,ich,ids,iel,ii1,iiii,incd,inspl,intra,
     *iroot,irr,itype1,j,j11,ja,jj1,l,l11,lay1,lay2,n1,nc,ncd,ndr,
     *nl,nnn,nrr
      common/work2/ yold,yint,vel,y1,
     *aindex,anyx,anyz,aux,aux1,aux2,auxx,auxz,avert,ay4,ay5,
     *px,pz,rc,v1,v2,vn,vrt,vvv,x2,xdif,yan,ydif,zint,zint1,
     *i,i11,iac,ich,ids,iel,ii1,iiii,incd,inspl,intra,
     *iroot,irr,itype1,j,j11,ja,jj1,l,l11,lay1,lay2,n1,nc,ncd,ndr,
     *nl,nnn,nrr
c ---
      integer*4 code,kref,kc,k_beg,k_end,k_step,k_sig
      common/cod/code(50),kref,kc,k_beg,k_end,k_step,k_sig
c ---
      real*8    a1,b1,c1,d1,x1,brd
      integer*4 iii,npnt,nint
      common/intrf/a1(n_intz,n_int),b1(n_intz,n_int),c1(n_intz,n_int),
     *d1(n_intz,n_int),x1(n_intz,n_int),brd(2),iii(n_intz,n_int),
     *npnt(n_int),nint
c ---
      real*8    ay,ds
      integer*4 kint,mreg,n,iref,ind,ind1
      common/ray/ay(12,10000),ds(11,50),kint(50),mreg,n,iref,ind,ind1
c ---
      integer*4 intr,int1,iout,kre,irefr,lay,itype,nder,iprint,mprint,ntr,imet
      common/auxi/intr,int1,iout,kre,irefr,lay,itype,nder,iprint,mprint,ntr,imet
c ---
      integer*4 nrho
      real*8    rho1,rho2
      common/den/rho1(n_lay),rho2(n_lay),nrho
c ---
      integer*4 icod,ivsp
      real*8    xvsp
      common/vsp/xvsp,icod,ivsp
c ---
c
      incd=0
      inspl=0
      n=n+1
      nrr=n
      ntr=1
      if(n.gt.10000)go to 50
c
c     Normalization of slowness vector
c
      yan=y(3)
      y(3)=101.d0
      call veloc(y,vel)
      y(3)=yan
      if(n.eq.1)go to 67
      vn=vel(1)
      vrt=vn*vn*(y(3)*y(3)+y(4)*y(4))
      vrt=1.d0/dsqrt(vrt)
      y(3)=y(3)*vrt
      y(4)=y(4)*vrt
c
c     Check of the position of the point of ray with respect to interface
c
      intr=lay
      nl=npnt(intr)-1
      ich=1
      if(y(1).lt.brd(1).or.y(1).gt.brd(2))go to 7
   65 ich=0
      ind=0
      do 2 i=1,nl
      j=i
      if(y(1).lt.x1(i+1,intr)) goto 3
    2 continue
    3 if(iref.eq.1)go to 27
      if(kint(iref-1).eq.(n-1))go to 21
   27 continue
      x2=x1(j,intr)
      aux=(y(1)-x2)/(x1(j+1,intr)-x1(j,intr))
      zint=a1(j,intr)+(6.0d0*aux*aux-15.0d0*aux+10.0d0)*
     xaux*aux*aux*(a1(j+1,intr)-a1(j,intr))
      zint1=zint
c
c     Upper intersection
c
      if(y(2).lt.zint) go to 9
      intr=lay+1
      nl=npnt(intr)-1
      do 4 i=1,nl
      j=i
      if(y(1).lt.x1(i+1,intr)) go to 5
    4 continue
    5 continue
      x2=x1(j,intr)
      aux=(y(1)-x2)/(x1(j+1,intr)-x1(j,intr))
      zint=a1(j,intr)+(6.0d0*aux*aux-15.0d0*aux+10.0d0)*
     xaux*aux*aux*(a1(j+1,intr)-a1(j,intr))
c CAUTION !!!! May be wrong
      if(zint1.gt.zint+1.0d-4) go to 54
c
c     Lower intercection
c
      if(y(2).gt.zint) go to 9
   21 if(y(1).le.brd(1).or.y(1).ge.brd(2))go to 7
c
c     Check whether the ray did not cross the borehole
c
      if(icod.eq.0)go to 67
      if(icod.gt.iref)go to 67
      aux=(ay(2,n-1)-xvsp)*(y(1)-xvsp)
      if(aux.gt.0.d0)go to 67
c
c     The ray crossed the borehole
c   
   69 ind=22
      avert=xvsp
      go to 33      
c
c     The ray did not cross any interface
c
   67 ay(1,n)=x
      do 6 i=1,4
    6 ay(i+1,n)=y(i)
      return
c
c     The ray crossed one of the vertical boundaries
c
    7 if(y(1).le.brd(1)) ind=1
      if(y(1).ge.brd(2)) ind=2
      avert=brd(ind)
   33 aux=y(1)-ay(2,n-1)
      if(dabs(aux).lt..00001d0)aux=0.d0
      if(dabs(aux).ge..00001d0)aux=(avert-ay(2,n-1))/aux
      x=ay(1,n-1)+aux*(x-ay(1,n-1))
      y(1)=avert
      do 8 i=2,4
    8 y(i)=ay(i+1,n-1)+aux*(y(i)-ay(i+1,n-1))
      if(ich.eq.1)go to 65
      ay(1,n)=x
      do 64 i=1,4
   64 ay(i+1,n)=y(i)
      prmt(5)=3.d0
      ds(2,iref)=y(3)
      ds(3,iref)=y(4)
      kint(iref)=n
      go to 49
c
c     The ray crossed an interface
c
c     The determination of the point of incidence
c
    9 yold(1)=ay(2,n-1)
      yold(2)=ay(3,n-1)
      irr=iref
      call root(yold,y,yint,anyx,anyz,rc,j,iroot)
c
c     Check whether the ray did not cross the borehole
c
      if(icod.eq.0)go to 12
      if(icod.gt.iref)go to 12
      aux=(yold(1)-xvsp)*(yint(1)-xvsp)
      if(aux.gt.0.d0)go to 12
c
c     The ray crossed the borehole
c   
      go to 69
   12 ind1=intr
      if(iroot.eq.100)go to 30
c
c     The loop for the iterative determination of the point of incidence
c     (not used in this version)
c
      if(iout.ne.0)go to 11
   10 n=n-1
      y1(1)=yint(1)
      y1(2)=yint(2)
      prmt(5)=1.1d0
      iout=1
      return
   11 iac=iac+1
      if(iac.eq.10)go to 30
      aux1=y1(1)-yint(1)
      aux2=y1(2)-yint(2)
      aux=aux1*aux1+aux2*aux2
      if(aux.gt..00001d0) go to 10
c
c     Determination of parameters of incident wave
c     at the point of incidence
c
   30 iac=0
      iout=0
      ay(2,n)=yint(1)
      ay(3,n)=yint(2)
      xdif=y(1)-ay(2,n-1)
      ydif=y(2)-ay(3,n-1)
      if(dabs(xdif).lt.dabs(ydif))go to 60
      if(dabs(xdif).lt..00001d0)go to 61
      aux=(ay(2,n)-ay(2,n-1))/xdif
      go to 62
   60 if(dabs(ydif).lt..00001d0)go to61
      aux=(ay(3,n)-ay(3,n-1))/ydif
      go to 62
   61 aux=0.d0
   62 continue
      ay(1,n)=ay(1,n-1)+aux*(x-ay(1,n-1))
      x=ay(1,n)
      ay5=(x-ay(1,n-1))/ay(6,n-1)
      ay4=ay(4,n-1)-ay(7,n-1)*ay5
      ay5=ay(5,n-1)-ay(8,n-1)*ay5
      y(1)=ay(2,n)
      y(2)=ay(3,n)
c
c     Normalization of slowness vector at the point of incidence
c
      call veloc(y,vel)
      vn=vel(1)
      vrt=ay4*ay4+ay5*ay5
      vrt=vn*vn*vrt
      vrt=1.d0/dsqrt(vrt)
      y(3)=ay4*vrt
      y(4)=ay5*vrt
      ay(4,n)=y(3)
      ay(5,n)=y(4)
c
      if(iref.eq.1)go to 49
      nnn=kint(iref-1)
      if(nnn.le.0)go to 49
      if(dabs(ay(2,nnn)-yint(1)).gt..00001d0.or.intr.ne.int1)go to 49
cxx   write(*,*) intr,int1,nnn,ay(2,nnn),yint(1)
      irr=irr-1
      ntr=2
      if(aindex.gt.1.0d0)go to 28
      ntr=3
      go to 26
c
   49 itype1=itype
      itype=1
      call veloc(y,vel)
      ds(4,iref)=vel(1)
      ds(8,iref)=rho1(lay)+rho2(lay)*vel(1)
      itype=-1
      call veloc(y,vel)
      ds(5,iref)=vel(1)
      itype=itype1
      vvv=ds(4,iref)
      if(itype.lt.0)vvv=ds(5,iref)
      ds(1,iref)=0.d0
      ds(6,iref)=0.d0
      ds(7,iref)=0.d0
      ds(9,iref)=0.d0
      ds(10,iref)=0.d0
      ds(11,iref)=0.d0
      ntr=4
      if(ind.eq.1.or.ind.eq.2.or.ind.eq.22)go to 32
      lay1=lay
      px=y(3)
      pz=y(4)
      yan=y(3)
      aux=anyx*px+anyz*pz
      if(aux.le.0.d0)go to 13
      anyx=-anyx
      anyz=-anyz
      aux=-aux
      rc=-rc
   13 auxx=(anyx*pz-anyz*px)*vvv
      auxz=aux*vvv
      ds(1,iref)=rc
      ds(2,iref)=auxx
      ds(3,iref)=auxz
      ntr=5
      if(kre.eq.1)go to 24
      if(kre.lt.1)go to 31
c
c     Multiply reflected wave
c
      ntr=6
      if((iref+1).gt.kre.and.intr.eq.int1)go to 26
      if((iref+1).gt.kre) go to 24
      ncd=code(iref+1)-code(iref)
      ntr=7
      if(kc.gt.0.and.iref.eq.1.and.intr.eq.lay)go to 26
      ntr=8
      if(kc.lt.0.and.iref.eq.1.and.intr.ne.lay)go to 26
      if(iref.eq.1)go to 46
      ntr=9
      if(intr.eq.int1.and.ncd.ne.0)go to 26
      if(intr.ne.int1.or.ncd.ne.0)go to 46
c
c    Compound element
c
      irefr=1
      kint(iref)=0
      iref=iref+1
      irr=iref
      ds(1,iref)=ds(1,iref-1)
      ds(2,iref)=ds(2,iref-1)
      ds(3,iref)=ds(3,iref-1)
      ds(4,iref)=ds(4,iref-1)
      ds(5,iref)=ds(5,iref-1)
      ds(6,iref)=0.d0
      ds(7,iref)=0.d0
      ds(8,iref)=ds(8,iref-1)
      ds(9,iref)=0.d0
      ds(10,iref)=0.d0
      ds(11,iref)=0.d0
      do 53 i=1,9
   53 ds(i,iref-1)=0.d0
      ncd=code(iref+1)-code(iref)
      ntr=10
      if((iref+1).gt.kre) go to 24
   46 int1=intr
      j11=j
      i11=intr
      l11=lay1
      if(ncd.ne.0) go to 22
c
c     Reflection of unconverted wave
c
      iiii=iii(j,intr)
      ntr=11
      if(iiii.eq.(-2))go to 48
      ntr=12
      if(iiii.gt.0)go to 23
      y(3)=101.d0
      call veloc(y,vel)
      y(4)=pz-2.d0*aux*anyz
      y(3)=px-2.d0*aux*anyx
      iref=iref+1
      v1=vel(1)
      v2=v1
      aindex=1.d0
      go to 20
c
c     Refracted wave
c
   31 ntr=13
      if(intr.eq.lay.and.lay.eq.1) go to 24
      ntr=14
      if(intr.gt.lay.and.intr.eq.nint) go to 24
      ncd=1
      int1=intr
      go to 14
c
c     Refraction or reflection of converted wave
c
   22 ncd=iabs(code(iref+1))-iabs(code(iref))
      iiii=iii(j,intr)
      ntr=15
      if(ncd.eq.0.and.iiii.eq.(-2)) go to 48
      ntr=16
      if(ncd.eq.0.and.iiii.gt.0)go to 23
   14 y(3)=101.d0
      call veloc(y,vel)
      y(3)=yan
      v1=vel(1)
      if(kre.gt.1)itype=isign(1,code(iref))
   45 iref=iref+1
c
c     Check whether a transmission takes place at an interface
c     which coincides with another interface
c
      if(ncd.eq.0.and.inspl.eq.0)go to 16
      if(ncd.eq.0.and.inspl.eq.1)go to 68
      if(intr.eq.lay) go to 15
      ntr=17
      if(ncd.lt.0)go to 26
      ntr=18
      if(kre.le.1.and.intr.eq.nint)go to 24
      ntr=19
      if(intr.eq.nint) go to 26
      lay=lay+1
      go to 29
   68 intr=intra
      j=ja
      incd=1
      go to 37
   15 ntr=20
      if(ncd.gt.0.and.kre.gt.1)go to 26
      ntr=21
      if(kre.le.1.and.lay.eq.1)go to 24
      ntr=22
      if(lay.eq.1)go to 26
      lay=lay-1
   29 intra=intr
      ja=j
      if(intr.eq.lay1)go to 36
      nc=npnt(intr+1)
      do 34 i=1,nc
      jj1=i
      ii1=iii(i,intr+1)
      if(j.eq.ii1)go to 35
   34 continue
      go to 16
   35 intr=intr+1
      j=jj1
      go to 37
   36 ii1=iii(j,intr)
      if(ii1.le.0)go to 16
      intr=intr-1
      j=ii1
c
c     Transmission at an interface which coincides
c     with another interface
c
   37 n=n+1
      n1=n+1
      ntr=23
      if(n1.ge.10000) go to 50
      inspl=1
      ndr=8
      if(nder.eq.1)ndr=11
      do 38 i=n,n1
      do 38 l=1,ndr
   38 ay(l,i)=ay(l,n-1)
      n=n1
      lay1=lay
      ind1=intr
      kint(iref)=-1
      do 47 i=1,11
   47 ds(i,iref)=0.d0
      ntr=24
      if(kre.gt.1.and.(code(iref)*itype).lt.0)go to 66
      ntr=25
      if(kre.gt.1.and.(iref+1).gt.kre)go to 24
      int1=intr
      if(kre.gt.1)ncd=iabs(code(iref+1))-iabs(code(iref))
      ntr=26
      if(ncd.eq.0.and.iii(j,intr).gt.0)go to 23
      go to 45
c
c     Determination of the angle of the ray of generated wave
c     at the point of incidence
c
   16 lay2=lay
      if(kre.gt.1)itype=isign(1,code(iref))
      call veloc(y,vel)
      v2=vel(1)
      if(incd.eq.1)ncd=0
      incd=0
      inspl=0
      aindex=v2/v1
      aux1=1.d0/(v2*v2)-1.d0/(v1*v1)+aux*aux
      if(aux1.gt.0.d0) go to 17
      ntr=27
      go to 28
   17 if(ncd.eq.0)auxz=-dsqrt(aux1)
      if(ncd.ne.0)auxz=dsqrt(aux1)
      aux1=auxz+aux
      auxz=auxz*v2
      y(3)=px-anyx*aux1
      y(4)=pz-anyz*aux1
c
c
c     Determination of parameters of generated wave
c     at the point of incidence
c
   20 prmt(5)=2.d0
      kint(irr)=nrr
      ds(1,irr)=rc
      ds(10,irr)=aindex*auxx
      ds(11,irr)=-auxz
      if(ncd.ne.0)go to 52
c
c     Determination of velocities and densities
c     on the opposite side of a reflector
c
      ja=j11
      intra=i11
      lay2=l11
      if(intra.eq.l11)go to 43
   40 lay2=lay2+1
      ntr=28
      if(intra.eq.nint.and.nder.ne.0)go to 51
      if(intra.eq.nint)return
      nc=npnt(intra+1)
      do 41 i=1,nc
      jj1=i
      ii1=iii(i,intra+1)
      if(ja.eq.ii1)go to 42
   41 continue
      go to 44
   42 intra=intra+1
      ja=jj1
      go to 40
   43 lay2=lay2-1
      ii1=iii(ja,intra)
      if(ii1.le.0)go to 44
      intra=intra-1
      ja=ii1
      go to 43
   44 if(lay2.ne.0)go to 52
      ds(6,irr)=0.d0
      ds(7,irr)=0.d0
      ds(9,irr)=0.d0
      return
   52 itype1=itype
      itype=1
      l11=lay
      lay=lay2
      call veloc(y,vel)
      ds(6,irr)=vel(1)
      ds(9,irr)=rho1(lay2)+rho2(lay2)*vel(1)
      itype=-1
      call veloc(y,vel)
      ds(7,irr)=vel(1)
      itype=itype1
      lay=l11
      return
c
c     Check whether the ray does not terminate at a fictitious interface
c
   24 ind=intr+100
      if(iref.eq.1.and.kc.gt.0.and.intr.eq.int1)go to 26
      if(iref.eq.1.and.kc.lt.0.and.intr.ne.int1)go to 26
c**      write(*,*)'INTRF',iref,kc,intr,int1
      if(intr.eq.1)ind=3
      if(intr.eq.nint)ind=4
      ntr=29
      if(iii(j,intr).eq.(-2))go to 48
      if(iii(j,intr).le.0)go to 39
      iel=j
      ids=intr
   63 ii1=iii(iel,ids)
      ntr=30
      if(iii(ii1,ids-1).eq.(-2)) go to 48
      if(iii(ii1,ids-1).le.0)go to 39
      iel=iii(ii1,ids-1)
      ids=ids-1
      go to 63
c
c     Termination of computation
c
   48 ind=16
      go to 25
   51 ind=15
      go to 39
   23 ind=17
      go to 39
   66 ind=18
      go to 39
   26 ind=8
      go to 39
   54 ind=20
      go to 39
   28 ind=9
   39 iref=irr
   25 prmt(5)=3.d0
      kint(irr)=nrr
      go to 32
   50 ind=13
      prmt(5)=3.d0
   32 if(ind.eq.9)return
      yan=y(3)
      y(3)=101.d0
      call veloc(y,vel)
      y(3)=yan
      return
      end
c**********************************************************************
c
c     Computation of the right-hand sides
c     of the ray tracing system equations
c
c**********************************************************************
      subroutine fct(x,y,dery)
      implicit none
c --- type definitions ---
      real*8    vel(6),y(4),dery(4),vv,v1,x
c --- commons ---
      integer*4 intr,int1,iout,kre,irefr,lay,itype,nder,iprint,mprint,ntr,imet
      common/auxi/intr,int1,iout,kre,irefr,lay,itype,nder,iprint,mprint,ntr,imet
c ---
      call veloc(y,vel)
      vv=vel(1)
      v1=1.d0/vv
      vv=vv*vv
      dery(1)=vv*y(3)
      dery(2)=vv*y(4)
      dery(3)=-v1*vel(2)
      dery(4)=-v1*vel(3)
      return
      end
c******************************************************************
c
c     Runge-Kutta metod for the ray computation
c    ===========================================
c ndim - dimension, (4)
c y(1) - x coordinate of starting point; y(2) - z coordinate;
c y(3) - p initial value, cos(f0)/v;    y(4) - sin(f0)/v
c prmt(1) - starting time; prmt(2) - maximal time;
c prmt(3) - step of integration;    prmt(4) - accuracy of integration;
c prmt(5) - step continuation:
c dery(4) - derivatives, right side of equations, dx/dt, dz/dt,
c           dp/dt and dq/dt accordingly
c ihlf - error codes:
c        11(ind=5) - in basic step time halved more then 10 times,
c                    and accuracy doesn't reached;
c        12(ind=6) - input step prmt(3)=0, not allowed;
c        13(ind=7) - prmt(3) < 0, not allowed.
c fct  - function for derivatives (dery) computation
c outpt - step analyzer function
c aux(8,4) - auxilary file:
c******************************************************************
      subroutine rkgs(prmt,y,dery,ndim,ihlf,aux)
      implicit none
c --- type description ---
      real*8    aj,bj,cj,delt,h,r1,r2,x,xend
      real*8    y(4),dery(4),aux(8,4),a(4),b(4),c(4),prmt(5)
      integer*4 i,iend,ihlf,imod,irec,istep,itest,j,ndim
c ---
      do 1 i=1,ndim
c-- 1.0/15.0=.0666667
    1 aux(8,i)=0.06666666666666667*dery(i)
      x=prmt(1)
      xend=prmt(2)
      h=prmt(3)
      prmt(5)=0.0
      call fct(x,y,dery)
      if(h*(xend-x))38,37,2
   2  a(1)=.5d0
c-- a(2)=(2.0-sqrt(2.0))/2.0=.2928932
      a(2)=0.2928932188134524756
c-- a(3)=(2.0+sqrt(2.0))/2.0=1.707107
      a(3)=1.7071067811865475244
c-- a(4)=1.0/6.0
      a(4)=0.1666666666666666667
      b(1)=2.d0
      b(2)=1.d0
      b(3)=1.d0
      b(4)=2.d0
      c(1)=.5d0
      c(2)=0.2928932188134524756
      c(3)=1.7071067811865475244
      c(4)=.5d0
      do 3 i=1,ndim
      aux(1,i)=y(i)
      aux(2,i)=dery(i)
      aux(3,i)=0.d0
    3 aux(6,i)=0.0d0
      irec=0
      h=h+h
      ihlf=-1
      istep=0
      iend=0
    4 if((x+h-xend)*h)7,6,5
    5 h=xend-x
    6 iend=1
    7 call outp(x,y,dery,irec,ndim,prmt)
      if(prmt(5))40,8,40
    8 itest=0
    9 istep=istep+1
      j=1
   10 aj=a(j)
      bj=b(j)
      cj=c(j)
      do 11 i=1,ndim
      r1=h*dery(i)
      r2=aj*(r1-bj*aux(6,i))
      y(i)=y(i)+r2
      r2=r2+r2+r2
   11 aux(6,i)=aux(6,i)+r2-cj*r1
      if(j-4)12,15,15
   12 j=j+1
      if(j-3)13,14,13
   13 x=x+.5d0*h
   14 call fct(x,y,dery)
      go to 10
   15 if(itest)16,16,20
   16 do 17 i=1,ndim
   17 aux(4,i)=y(i)
      itest=1
      istep=istep+istep-2
   18 ihlf=ihlf+1
      x=x-h
      h=.5d0*h
      do 19 i=1,ndim
      y(i)=aux(1,i)
      dery(i)=aux(2,i)
   19 aux(6,i)=aux(3,i)
      go to 9
   20 imod=istep/2
      if(istep-imod-imod)21,23,21
   21 call fct(x,y,dery)
      do 22 i=1,ndim
      aux(5,i)=y(i)
   22 aux(7,i)=dery(i)
      go to 9
   23 delt=0.d0
      do 24 i=1,ndim
   24 delt=delt+aux(8,i)*dabs(aux(4,i)-y(i))
      if(delt-prmt(4))28,28,25
   25 if(ihlf-10)26,36,36
   26 do 27 i=1,ndim
   27 aux(4,i)=aux(5,i)
      istep=istep+istep-4
      x=x-h
      iend=0
      go to 18
   28 call fct(x,y,dery)
      do 29 i=1,ndim
      aux(1,i)=y(i)
      aux(2,i)=dery(i)
      aux(3,i)=aux(6,i)
      y(i)=aux(5,i)
   29 dery(i)=aux(7,i)
      call outp(x-h,y,dery,ihlf,ndim,prmt)
      if(prmt(5))40,30,40
   30 do 31 i=1,ndim
      y(i)=aux(1,i)
   31 dery(i)=aux(2,i)
      irec=ihlf
      if(iend)32,32,39
   32 ihlf=ihlf-1
      istep=istep/2
      h=h+h
      if(ihlf)4,33,33
   33 imod=istep/2
      if(istep-imod-imod)4,34,4
   34 if(delt-.02d0*prmt(4))35,35,4
   35 ihlf=ihlf-1
      istep=istep/2
      h=h+h
      go to 4
   36 ihlf=11
      call fct(x,y,dery)
      go to 39
   37 ihlf=12
      go to 39
   38 ihlf=13
   39 call outp(x,y,dery,ihlf,ndim,prmt)
   40 return
      end
