c******************************************************************
c
c     Ray tracing by the Runge-Kutta method, subsidary tracing
c     
c     x0,z0 - initial point of the ray
c     t0    - initial time
c     fi0   - initial angle
c     x,z   - point at which the ray computation is terminated
c     t,fi  - corresponding time and angle
c     tmax  - the maximum permitted time, here tmax=10000.
c     dt    - step of integration
c     ac    - accuracy of integration, recommended (0.0001 -0.001 ??)
c-------- control parameters ---------------------------------------
c     ind/ray/ - starting layer
c     kc/cod/  - =1, should strike interface below the source
c     kref/cod/ - the number of simple element in the ray
c******************************************************************
      subroutine ray3(x0,z0,t0,fi0,x,z,t,fi,tmax,dt,ac,nn)
      implicit none
c ---
      integer*4 n_int,n_intz
      parameter (n_int=20,n_intz=50)
c --- type definitions ---
      real*8    y(4),dery(4),din(4),prm(5),aux(8,4),vel(6)
      real*8    ac,cs,dt,fi,fi0,sn,t,t0,tmax,v,x,x0,z,z0
      integer*4 i,ihlf,isour,nn
c --- commons ---
      real*8    a1,b1,c1,d1,x1,brd
      integer*4 iii,npnt,nint
      common/intrf/a1(n_intz,n_int),b1(n_intz,n_int),c1(n_intz,n_int),
     *d1(n_intz,n_int),x1(n_intz,n_int),brd(2),iii(n_intz,n_int),
     *npnt(n_int),nint
c ---
      integer*4 intr,int1,iout,kre,irefr,lay,itype,nder,iprint,mprint,ntr,imet
      common/auxi/intr,int1,iout,kre,irefr,lay,itype,nder,iprint,mprint,ntr,imet
c ---
      real*8    ay,ds
      integer*4 kint,mreg,n,iref,ind,ind1
      common/ray/ay(12,10000),ds(11,50),kint(50),mreg,n,iref,ind,ind1
c ---
      integer*4 code,kref,kc,k_beg,k_end,k_step,k_sig
      common/cod/code(50),kref,kc,k_beg,k_end,k_step,k_sig
c ---
      y(1)=x0
      y(2)=z0
      irefr=0
      kre=kref
      if(kc.eq.0)kre=0
      n=nn
      iref=1
      iout=0
      isour=ind
c
c     Determination of initial conditions for the runge-kutta procedure
c
      lay=isour
      if(isour.ne.iabs(code(1)))ind=14
      if(isour.ne.iabs(code(1)))return
      itype=isign(1,code(1))
      y(3)=0.d0
      y(4)=0.d0
      call veloc(y,vel)
      v=vel(1)
      y(3)=dcos(fi0)/v
      y(4)=dsin(fi0)/v
      ind=0
      ind1=0
      prm(1)=t0
      prm(2)=tmax
      prm(3)=dt
      prm(4)=ac
      t=prm(1)
      do 21 i=1,4
   21 din(i)=.25d0
    9 do 10 i=1,4
   10 dery(i)=din(i)
c
c     Computation of the ray
c
      call rkgs(prm,y,dery,4,ihlf,aux)
      if(ihlf.eq.11)ind=5
      if(ihlf.eq.12)ind=6
      if(ihlf.eq.13)ind=7
      if(ind.ge.5.and.ind.le.7)return
      if(ind.eq.20)return
      if(ind.eq.21)return
      if(dabs(prm(5)).gt..0001d0) go to 11
c
c     Interuption of the computation of the ray due to (t.ge.prm(2))
c     if prm(2).eq.tmax - termination of the computation
c     if prm(2).ne.tmax - the computation continues further on with
c     the regular time step along the ray
c
      t=ay(1,n)
      if(t.ge.tmax)go to 15
      go to 14
   11 if(dabs(prm(5)-1.1d0).gt..0001d0) go to 13
c
c     Loop for the iterative determination of the point of incidence
c     (not used in this version)
c
      prm(1)=ay(1,n)
      prm(3)=prm(3)/(2.d0**(ihlf+1))
      do 12 i=1,4
   12 y(i)=ay(i+1,n)
      t=ay(1,n)
      n=n-1
      go to 9
   13 x=y(1)
      z=y(2)
      t=ay(1,n)
      if(dabs(prm(5)-2.d0).gt..0001d0.and.irefr.eq.1)ind1=-ind1
      if(dabs(prm(5)-2.d0).gt..0001d0)go to 20
c
c     Integration from the point of reflection/transmission
c     to the closest point of the ray corresponding to the regular
c     time step along the ray
c
      prm(1)=ay(1,n)
      i=int((prm(1)-t0)/dt)
      prm(2)=float(i+1)*dt+t0
      go to 9
   14 prm(1)=prm(2)
      prm(2)=tmax
      prm(3)=dt
      n=n-1
      go to 9
   15 ind=12
      x=y(1)
      z=y(2)
      if(irefr.eq.1)ind1=-ind1
c
   20 v=ay(6,n)
      cs=y(3)*v
      sn=y(4)*v
      fi=datan2(sn,cs)
      return
      end
