c=========================================================
c Program: conv_cu_idc_n, Autor: M.P.Barmin, Date: 02/22/01
c Program designed to convert CU SSSCs from polar coordinate
c system to geographical with the step 1x1 degree.
c Program subtracts from SSSC's station elevation correction,
c provides antialiasing and produce zero uncertenties.
c=========================================================
      implicit none
c --- common's --------------------------------------
      real*4    flat,flon,elev,tab_cu(101,120),unc_cu(101,120)
      character*8 sta,sta_surr,pha
      integer*4 max_delta
      common/cu/flat,flon,elev,max_delta,sta,sta_surr,pha,tab_cu,unc_cu
c ---
      real*4    t_iasp91(101)
      common/iasp91/t_iasp91
c ---
      real*4    rad,deg,sig1,sig2
      common/par/rad,deg,sig1,sig2
c --- type def. -----------------
      character*8 station,phase
      character*160 in_dir,out_file,iasp_file, mod_nam
      integer*4 narg,iargc
      integer*4 i,j,l,n,ierr
      integer*4 ilat1,ilat2,ilon1,ilon2
      real*4    rad,t,fla,flo,az,del,deg,p_lim(6),glim,err,elev1,e_corr,depth
      character*8 p_list(6)
c ---
      data p_list/'Pg','Pn','P','Lg','Sn','S'/
      data p_lim /9.0,22.0,22.0,9.0,22.0,22.0/
      rad=atan(1.0)/45.0
      deg=rad*6371.0
c replace this values, if you'd like that ==========
      sig1=30.0
      sig2=100.0
c getting command line parameters =======================
      narg=iargc()
      if(narg.ne.8)
     * STOP
     * 'Usage: conv_cu_idc_n station phase elev depth in_dir iasp91_tbl_file mod_name out_file'
      call getarg(1,station)
      call getarg(2,phase)
      call getarg(3,in_dir)
      read(in_dir,*) elev1
      call getarg(4,in_dir)
      read(in_dir,*) depth
      call getarg(5,in_dir)
      call getarg(6,iasp_file)
      call getarg(7,mod_nam)
      call getarg(8,out_file)
c checking for wrong phase ===================
      do i=1,6
        if(phase.eq.p_list(i)) then
          n=i
          glim=p_lim(i)+0.5
          goto 1
        endif
      enddo
      write(*,1000) phase,(p_list(l),l=1,6)
      stop
c reading elevation coefficients ================
    1 call r_iasp91_tbl(iasp_file,p_list(n),depth,ierr)
c reading CU SSSC's =============================
      call r_cu_sssc(station,phase,depth,in_dir,mod_nam)
      call uncert(n)
c making a square window around station =========
      ilat1=flat-glim
      ilat2=flat+glim
      ilon1=flon-glim/cos(flat*rad)
      ilon2=flon+glim/cos(flat*rad)
      if(ilat2.gt.90.0) then
        ilat2=90.
        ilon1=0.
        ilon2=359.0
      endif
      if(ilat1.lt. -90.) then
        ilat1=-90.
        ilon1=0.
        ilon2=359.0
      endif
      write(*,1001) station,p_list(n),ilat1,ilat2,ilon1,ilon2
      open(8,file=out_file,status='unknown')
c conversation polar grid to Cartesian 1x1 grid ==============
      do i=ilat1,ilat2
        fla=i
        do j=ilon1,ilon2
          flo=j
          call delaz(flat,flon,fla,flo,del,az)
          call gauss(del,az,t,err,e_corr)
c Important thing. Elevation correction is subtacted from time !!!!
          t=t+e_corr*elev1
          write(8,'(4f9.3)') fla,flo,t,err
        enddo
      enddo
      close(8)
 1000 format('(E200) Phase ',a3,'is not found, available phases are: ',
     *5(a2,','),a2)
 1001 format('(I201) Sta: ',a4,', Phase: ',a2,
     *', Limits: ',2i5,' (lat), ',2i5,' (lon)')
      end
c======================================================
c gauss  -smoothing SSSc 
c======================================================
      subroutine gauss(del,az,t,err,e_corr)
      implicit none
c --- common's --------------------------------------
      real*4    flat,flon,elev,tab_cu(101,120),unc_cu(101,120)
      character*8 sta,sta_surr,pha
      integer*4 max_delta
      common/cu/flat,flon,elev,max_delta,sta,sta_surr,pha,tab_cu,unc_cu
c ---
      real*4    rad,deg,sig1,sig2
      common/par/rad,deg,sig1,sig2
c ---
      real*4    t_iasp91(101)
      common/iasp91/t_iasp91
c ---
      real*4 del,az,t,err,se2,se3,e_corr
      real*4 sig1,sig2,d1,d2,s1,s2,dco,dsi,aa,dd,w1,w2,ddd,ss,ddr
      integer*4 i,j,id1,id2,ic
c ---
      d1=del-sig2
      if(d1.le.0.0) d1=0.0
      id1=(d1+5.0)/25.0+1
      d2=del+sig2
      id2=(d2+5.0)/25.0+1
      if(id2.gt.max_delta+1) id2=max_delta+1
c finding points ===============
      ic=0
      s1=0.0
      s2=0.0
      se2=0.0
      se3=0.0
      dco=del*cos(az*rad)
      dsi=del*sin(az*rad)
      do i=1,120
        aa=(i-1)*3.0
        do j=id1,id2
          dd=(j-1)*25.0
          w1=dd*cos(aa*rad)
          w2=dd*sin(aa*rad)
          ddr=sqrt(w1*w1+w2*w2)/6371.0
          w1=w1-dco
          w2=w2-dsi
          ddd=sqrt(w1*w1+w2*w2)
          if(ddd.le.sig2) then
          ic=ic+1
          ss=exp(-ddd*ddd/sig1/sig1/2.0)*ddr
          s1=s1+ss
          s2=s2+tab_cu(j,i)*ss
          se2=se2+unc_cu(j,i)*ss
          se3=se3+t_iasp91(j)*ss
          endif
        enddo
      enddo
      t=s2
      err=se2
      e_corr=se3
      if(ic.eq.0)  return
      t=s2/s1
      err=se2/s1
      e_corr=se3/s1
      return
      end
c======================================================
c uncert - modeling errors   
c======================================================
      subroutine uncert(n)
      implicit none
c --- common's --------------------------------------
      real*4    flat,flon,elev,tab_cu(101,120),unc_cu(101,120)
      character*8 sta,sta_surr,pha
      integer*4 max_delta
      common/cu/flat,flon,elev,max_delta,sta,sta_surr,pha,tab_cu,unc_cu
c ---
      real*4    rad,deg,sig1,sig2
      common/par/rad,deg,sig1,sig2
c --- type def. -----------------
      integer*4 n,i,j,id
      real*4    P_err(21),S_err(21),dh,dis
c ---
      data P_err/21*0.0/
      data S_err/21*0.0/
      do i=1,120
        do j=1,max_delta+1
         dis=(j-1)*25.0/deg
         id=dis
         id=id+1
         if(id.le.1) id=1
         if(id.gt.20) id=20
         dh=dis-float(id-1)
         if(n.le.3) then
            unc_cu(j,i)=(P_err(id+1)-P_err(id))*dh+P_err(id)
         else
            unc_cu(j,i)=(S_err(id+1)-S_err(id))*dh+S_err(id)
         endif
        enddo
      enddo
      return
      end
c=============================================================
c r_iasp91_tbl - reading IASP91 station elevation coefficients
c=============================================================
      subroutine r_iasp91_tbl(f_name,ph,depth,ierr)
      implicit none
c --- type definitions ---
      character*160 f_name
      character*8 phase,ph,sta1,sta2
      real*4    dum(20),hhh(20),res(20),depth
      integer*4 i,k,ij,kk,l,ll,ierr,nhs
c ---
      real*4    t_iasp91(101)
      common/iasp91/t_iasp91
c ---
      real*4    flat,flon,elev,tab_cu(101,120),unc_cu(101,120)
      character*8 sta,sta_surr,pha
      integer*4 max_delta
      common/cu/flat,flon,elev,max_delta,sta,sta_surr,pha,tab_cu,unc_cu
c ---
      ierr=0
      open(7,file=f_name,status='old')
      do kk=1,6
      read(7,'(4f12.5,2i5,1x,a8,1x,a8,1x,a8)')
     *      (dum(l),l=1,4),nhs,max_delta,sta1,sta2,phase
      read(7,*) (hhh(l),l=1,nhs)
        do i=1,max_delta+1
          read(7,*) (dum(ll),ll=1,nhs+1),(res(l),l=1,nhs)
          k=0
          do ij=2,nhs
            if(depth.ge.hhh(ij-1).and.depth.le.hhh(ij)) then
                k=ij-1
                goto 2
            endif
          enddo
          write(*,*) 'Wrong depth = ',depth, ', program conv_cu_idc aborted'
    2     if(res(k+1).gt.9000.0.or.res(k).gt.9000.0) then
             t_iasp91(i)=0.0
             goto 3
          endif
          t_iasp91(i)=(res(k+1)-res(k))/(hhh(k+1)-hhh(k))*(depth-hhh(k))+res(k)
    3     continue
        enddo
        if(phase.eq.ph) then
          close(7)
          return
        endif
      enddo
      ierr=1
      write(*,'("(E202) phase not found: ",a8)') ph
      close(7)
      return
      end
c======================================================
c r_cu_sssc - read CU SSSC's
c======================================================
      subroutine r_cu_sssc(station,phase,depth,in_dir,mod_nam)
      implicit none
c --- common's --------------------------------------
      real*4    flat,flon,elev,tab_cu(101,120),unc_cu(101,120)
      character*8 sta,sta_surr,pha
      integer*4 max_delta
      common/cu/flat,flon,elev,max_delta,sta,sta_surr,pha,tab_cu,unc_cu
c --- type def. -----------------
      character*8 station,phase
      character*160 in_dir,file, mod_nam
      integer*4 i,j,ij,k,l,ll,l1,l2,l3,iaz,idel,iflag,lnblnk,nhs
      real*4    t,az,del,dum(6)
      real*4   depth,hhh(20),res(20)
c ---
      l=lnblnk(in_dir)
      l1=lnblnk(station)
      l2=lnblnk(phase)
      l3=lnblnk(mod_nam)
      file=in_dir(1:l)//'/'//station(1:l1)//'.'//phase(1:l2)//'.'//mod_nam(1:l3)//'.dat'
c clearing SSSCs ===========================
      do i=1,120
        do j=1,max_delta+1
          tab_cu(j,i)=0.0
        enddo
      enddo
c reading SSSCs
      open(7,file=file,status='old')
      iflag=0
      do i=1,120
        if(iflag.eq.0) then
          read(7,1000,end=9) az,flat,flon,elev,nhs,max_delta,sta,sta_surr,pha
          read(7,*) (hhh(ll),ll=1,nhs)
          iflag=1
        else
          read(7,1000,end=9) az
          read(7,*) dum(1)
        endif
        iaz=(az+1.0)/3.0 +1
        do j=1,max_delta+1
          read(7,1001,end=9) del,dum(1),(res(l),dum(2),l=1,nhs)
          k=0
          do ij=2,nhs
            if(depth.ge.hhh(ij-1).and.depth.le.hhh(ij)) then
                k=ij-1
                goto 2
            endif
          enddo
          write(*,*) 'Wrong depth = ',depth, ', program conv_cu_idc aborted'
    2     if(res(k+1).gt.9000.0.or.res(k).gt.9000.0) then
             t=0.0
             goto 3
          endif
          t=(res(k+1)-res(k))/(hhh(k+1)-hhh(k))*(depth-hhh(k))+res(k)
    3     idel=(del+12.0)/25.0 +1
          tab_cu(idel,iaz)=t
        enddo
      enddo
    9 close(7)
      return
 1000 format(4f12.5,2i5,1x,a8,1x,a8,1x,a8)
 1001 format(f7.1,16f9.3)
      end
