c========================================================
c time_iasp91s designed for calculation Sg/Sn/S or minimal
c travel time for IASP91 1-D model
c Input:
c phasec  - input phase S, Sn, Sg or SM (minimal travel time)
c p_hc    - S
c del     - distance, (km), del >= 0.0
c h       - depth, (km), depth >= 0.0
c Output:
c t       - travel time, (s)
c dtd     - derivatie dd/dt, (km/s), 1/slowness
c ierr    - 0 -OK, !=0 - error
c========================================================
      subroutine time_iasp91s(phasec,p_hc,del,h,t,dtd,ierr)
      implicit none
c --- Type definitions ---
      character*8 phase,p_h
      character*8 phasec,p_hc
      real*4      del,h,t,dtd,d_rad
      integer *4  ierr
c ---
      integer*4 max
      parameter (max=60)
      logical prnt(3)
      character*8 phcd(max),phlst(10)
      character*160 modnam
      real*4    tt(max),dtdd(max),dtdh(max),dddp(max)
      real*4    usrc(2)
      integer*4 in,n,i,ifirst
      real*4    zs,delta
      character*160 ename, evalue
c ---
cxx      data in/1/,modnam/'/home/barmin/DATA/ak135_cubic/ak135'/,
      data in/1/,modnam/'./iasp91'/,
     *     phlst(1)/'S+'/,prnt(1)/.false./,prnt(2)/.false./,
     *     prnt(3)/.true./,ifirst/0/
c ---
      phase=phasec
      p_h=p_hc
      ierr=0
      t=0.0
      dtd=0.0
      if(ifirst.eq.0) then
      ename='IASPEI91_TABLE'
      call getenv (ename, evalue)
      modnam=evalue
      d_rad=6371.0*atan(1.0)/45.0
        phlst(1)=p_h(1:1)//'+'
        call tabin(in,modnam)
        call brnset(1,phlst,prnt)
        ifirst=1
      endif
c
      if(h.lt.0.0) go to 1
        zs=h
        call depset(zs,usrc)
      delta=del
      delta=delta/d_rad
      if(delta.lt.0.) go to 2
      call trtm(delta,max,n,tt,dtdd,dtdh,dddp,phcd)
      if(n.le.0) go to 3
c =============================================
c     do 5 i=1,n
c     if(i.gt.1) go to 6
c     write(*,100)delta,i,phcd(i),tt(i),dtdd(i),dtdh(i),
c    1 dddp(i)
c     go to 5
c6    write(*,102)i,phcd(i),tt(i),dtdd(i),dtdh(i),dddp(i)
c5    continue
c =============================================
      if(phase.eq.'SM') then
        t=tt(1)
        dtd=d_rad/dtdd(1)
        return
      else if(phase.eq.'S') then
        do i=1,n
          if(phcd(i).eq.phase) then
            t=tt(i)
            dtd=d_rad/dtdd(i)
            return
          endif
        enddo
        goto 3
      else if(phase.eq.'Sn') then
        do i=1,n
          if(phcd(i).eq.phase) then
          dtd=d_rad/dtdd(i)
c     write(*,*) 'R',phcd(i),i,t,dtd
            t=tt(i)
            return
          endif
        enddo
      else if(phase.eq.'Sg') then
        do i=1,n
          if(phcd(i).eq.'Sg'.or.phcd(i).eq.'Sb') then
            t=tt(i)
            dtd=d_rad/dtdd(i)
            return
          endif
        enddo
      endif
      goto 3
c   h < 0 --- ierr=1
   1  ierr=1
      return
c   delta < 0 --- ierr=2
   2  ierr=2
      return
c   No any solution --- ierr=3
   3  ierr=3
      return
 100  format(/1x,f6.2,i5,2x,a,f9.2,1p3e11.2)
 101  format(/1x,'No arrivals for delta =',f7.2)
 102  format(7x,i5,2x,a,f9.2,1p3e11.2)
      end
c=============================
c Test
c=============================
c     character*8 phase,p_h
c 1   write(*,*) 'Enter phase,p_h,del,h ==>'
c     read(*,*) phase,p_h,del,h
c     call time_iasp91s(phase,p_h,del,h,t,dtd,ierr)
c     write(*,*) phase,p_h,del,h,t,dtd,ierr
c     goto 1
c     end
