c==========================================================
c calculation intermediate points across the big arc
c on a sphere with fixed step by epicentral distance
c ising starting point f0,l0, and azimith azi.
c input: step - step along arc (deg)
c        f0,l0 - latitude and longitude of the 
c                beginning point (deg)
c        azi   - azimuth for direction (deg)
c         m     - number of points
c output: delta - distance between first and last points (deg)
c         ff,ll - point's coordinate (lat,lon) (deg)
c===========================================================
      subroutine path_az(step,f0,l0,azi,delta,m,ff,ll)
      implicit none
c --- type definitions ---
      dimension ff(100),ll(100)
      integer*4 i,m
      real*8    al,ang,anorm,bang,bl,del,delta,f0,ff,l0,ll
      real*8    scal,st,step,x0,x1,xx,y0,y1,yy,z0,z1,zz
      real*8    ax(3),ay(3),x(3),y(3),d,azi,azim,f0_c
c ---
      real*8    geo,r0,pi,rad
      common/const/geo,r0,pi,rad
c ---
      st=step*rad
      azim=azi*rad
      f0_c=f0
      x0=dcos(l0*rad)*dcos(f0_c*rad)
      y0=dsin(l0*rad)*dcos(f0_c*rad)
      z0=dsin(f0_c*rad)
c create azimuthal coordinate system ax, ay ===========
      ax(1)=0.0d0
      ax(2)=0.0d0
      ax(3)=1.0d0
      x(1)=x0
      x(2)=y0
      x(3)=z0
      call nvect(ax,x,y,ay,d)
      call nvect(x,ay,y,ax,d)
      do i=1,3
        y(i)=ax(i)*dcos(azim)+ay(i)*dsin(azim)+x(i)
      enddo
      d=dsqrt(y(1)*y(1)+y(2)*y(2)+y(3)*y(3))
      x1=y(1)/d
      y1=y(2)/d
      z1=y(3)/d
      scal=x0*x1+y0*y1+z0*z1
      del=dacos(scal)
      do i=1,m
        ang=st*(i-2)
        bang=del-ang
        al=(dcos(ang)-scal*dcos(bang))/(1-scal*scal)
        bl=(dcos(bang)-scal*dcos(ang))/(1-scal*scal)
        xx=al*x0+bl*x1
        yy=al*y0+bl*y1
        zz=al*z0+bl*z1
        anorm=dsqrt(xx*xx+yy*yy+zz*zz)
        xx=xx/anorm
        yy=yy/anorm
        zz=zz/anorm
        ff(i)=dasin(zz)/rad
        ll(i)=datan2(yy,xx)/rad
      enddo
      delta=del/rad
      return
      end
c******************************************************
      subroutine nvect(a,b,c,cn,d)
      implicit none
      real*8    d,a(3),b(3),c(3),cn(3)
      integer*4 i
      c(1)=a(2)*b(3)-a(3)*b(2)
      c(2)=a(3)*b(1)-a(1)*b(3)
      c(3)=a(1)*b(2)-a(2)*b(1)
      d=dsqrt(c(1)*c(1)+c(2)*c(2)+c(3)*c(3))
      if(d.eq.0.0d0) then
      if(a(1).ne.0.0d0) then
      cn(1)=-a(2)
      cn(2)=a(1)
      cn(3)=a(3)
      else
      cn(1)=a(1)
      cn(2)=-a(3)
      cn(3)=a(2)
      endif
      d=dsqrt(cn(1)*cn(1)+cn(2)*cn(2)+cn(3)*cn(3))
      do 10 i=1,3
   10 cn(i)=cn(i)/d
      d=0.0d0
      return
      endif
      do 20 i=1,3
   20 cn(i)=c(i)/d
      return
      end
c=============================================
c test
c=============================================
cxx   real*4 ff(100),fll(100)
cxx1  read(*,*,end=2) df,dl,az,step,n
cxx   call path_az(step,df,dl,az,delta,n,ff,fll)
cxx   write(*,*) df,dl,az,step,delta,n
cxx   do i=1,n
cxx     write(*,*) ff(i),fll(i)
cxx   enddo
cxx   goto 1
cxx2  continue
cxx   end
